#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np

def rotation_matrix(omega):
    """Calculates the classical 2D-rotation matrix with angle ``omega``.

    Args:
        omega (float): Rotation-Angle

    Returns:
        np.ndarray: The resulting rotation matrix of dimension (2, 2).
    """
    # TODO
    #alpha = np.radians(omega)
    alpha = omega
    r = np.array([[np.cos(alpha), -np.sin(alpha)], [np.sin(alpha), np.cos(alpha)]])
    return r


def reflection_matrix(omega):
    """Calculates the classical 2D-reflection matrix with angle ``omega``.

    Args:
        omega (float): Reflection-Angle

    Returns:
        np.ndarray: The resulting reflection matrix of dimension (2, 2).
    """
    # TODO
    alpha = 2 * omega
    r = np.array([[np.cos(alpha), np.sin(alpha)], [np.sin(alpha), -np.cos(alpha)]])
    return r


def eye(n, m):
    """Determines an identity matrix, i.e. a matrix with ones on its diagonal and
    zeros everywhere else, of arbitrary shape.

    Args:
        n (int): Number of Rows
        m (int): Number of Columns

    Returns:
        np.ndarray: The resulting "identity matrix" of dimension (n, m).
    """
    # TODO
    return np.eye(n, m, dtype = int)


def compose(*matrices):
    """Composes the given ``matrices`` to one single matrix.

    Args:
        matrices (List[np.ndarray]): Input matrices.

    Returns:
        np.ndarray: Composed Matrix.
    """
    assert matrices, "Expected at least one Matrix"
    # TODO
    #result = eye(matrices[0].shape)
    #for m in matrices:
    #    result = np.dot(result, m)
    for i in range(len(matrices)):
        if i == 0:
            result = matrices[i]
        else:
            result = np.dot(result, matrices[i])
    return result


def antidiag(values):
    """Calculates an antidiagonal matrix, i.e. a matrix where the entries are
    filled on the diagonal from bottom left to top right.

    Args:
        values (List[float]): The values on the antidiagonal.

    Returns:
        np.ndarray: The resulting quadratic matrix.

    Example:
        ``antidiag([2., 3.])`` should return the matrix
        ``[[0., 3.], [2., 0.]]``. So please fill the values from bottom left
        to top right.
    """
    assert len(values) >= 1
    # TODO
    len_l = len(values)
    # wieso funktionniert es nicht wenn 
    # arr = np.array((len_l, len_l)) eer aufgeufen wird ?
    arr = np.ndarray((len_l, len_l))
    k = 0
    for i in range(len_l - 1, -1, -1):
        for j in range(len_l):
            if i + j != len_l - 1:
                arr[i, j] = 0
            else:
                arr[i, j] = values[k]
                k += 1
    return arr


def vandermonde_matrix(values):
    """Generates a Vandermonde Matrix.

    Args:
        values (List[float]): Input values.

    Returns:
        np.ndarray: A quadratic matrix filled with the powers of the input
        values increasing from left to right columnwise.
    """
    assert len(values) >= 1
    # TODO
    len_l = len(values)
    arr = np.ones((len_l, len_l))
    for j in range(1, len_l):
        for i  in range(len_l):
            arr[i, j] = values[i] ** j
    return arr

def kronecker_product(A, B):
    """Calculates the Kronecker-Matrix-Product of ``A`` and ``B``.

    Args:
        A (np.ndarray): First Factor.
        B (np.ndarray): Second Factor.

    Warning:
        ``A`` as well as ``B`` might be multidimensional, i.e.
        ``len(shape(A))`` is **not** bounded from above!

    Returns:
        np.ndarray: The resulting matrix.
    """
    m, n = A.shape
    p, q = B.shape
    result = np.zeros((m * p, n * q))
    for i in range(m):
        for j in range(n):
            result[i * p : (i + 1) * p, j * q : (j + 1) * q] = A[i, j] * B
    return result


def walsh_matrix(n):
    """Calculates the Walsh Matrix with index ``n``.

    Args:
        n (int): Determines the dimensions and contents of the Walsh-Matrix

    Returns:
        np.ndarray: The resulting matrix of dimension (2**n, 2**n).
    """
    # TODO
    result = np.ones((1, 1))
    for i in range(1, n + 1):
        if i == 1:
            arr_1 = np.ones((2, 2))
            arr_1[1, 1] = -1
            result = 1 * arr_1
        else:
            result = kronecker_product(arr_1, walsh_matrix(n - 1))
    return result


def main():
    """The Main-Function of the programme. Is executed whenever this file is
    executed at top level.

    Hier kann beliebiger Testcode stehen, der bei der Korrektur vollständig
    ignoriert wird.
    """
    print(eye(3, 4))
    print(vandermonde_matrix([2.0, 4.0, 5.0]))


if __name__ == "__main__": main()