#!/usr/bin/env python3

def contains_char(string, char):
    """Checks if ``char`` is contained in the string ``string``. Returns True,
    if ``char`` is contained, and False otherwise.

    Args:
        string (str):
            The string that ``char`` is to be searched in.
        char (str):
            The character that is to be searched in ``string``. You may
            safely assume, that ``len(char) == 1`` holds the entire time.

    Returns:
        bool:
            True iff ``char`` is contained in ``string`` and False otherwise.
    """
    assert len(char) == 1
    # TODO
    for c in string:
        if c == char:
            return True
    return False


def is_palindrome(string):
    """Checks if ``string`` is a palindrome, i.e. is the same when read from
    left to right as well as right to left.

    Args:
        string (str):
            The string that should be a palindrome. Possible whitespaces
            (``\n``,``\t``,`` ``,``\r``) are to be removed from the string
            before checking if its a palindrome.

    Returns:
        bool:
            True iff ``string`` is a palindrome, False otherwise.
    """
    # TODO
    characters = "\n\t\t "
    for c in characters:
        string = string.replace(c, "")
    reverse_string = string[::-1]
    if string == reverse_string:
        return True
    else:
        return False


def count_char_frequency(string):
    """Counts the frequency of the occuring characters in ``string``.

    Args:
        string (str):
            The string of which the character frequency is counted.

    Returns:
        Dict[str, int]:
            A dictionary containing the characters as keys and
            the number of occurences in ``string`` as values.
    """
    # TODO
    dic = {}
    for c in string:
        if dic.get(c) == None:
            dic[c] = 1
        else:
            dic[c] += 1
    return dic


def first_non_repeating_char(string, repeating=False):
    """Returns the first non repreating character of the string ``string``, if
    there is one. Otherwise returns None. If ``repeating`` is equal to ``True``
    this function returns the first repeating character of the string
    ``string``.

    Args:
        string (str):
            The string of which the first (non) repeating character is
            returned.
        repeating (bool, *optional*=False):
            If this is set to ``True``, then the first repeating char is
            returned.

    Returns:
        Optional[str]:
            The first (non) repeating character of the string ``string`` or
            None, if it does not exist. It should hold, that whenever the
            return value is not equal to ``None``, it has a length of 1.
    """
    # TODO
    dic = count_char_frequency(string)
    if repeating == True:
        for c in string:
            if dic[c] > 1:
                return c
    else:
        for c in string:
            if dic[c] == 1:
                return c


def rotate_string(string, left_rot, right_rot):
    """Rotates the given ``string`` by ``left_rot`` to the left and
    ``right_rot`` to the right.

    Args:
        string (str):
            The string that is to be rotated.
        left_rot (int):
            The amount of characters the string is to be rotated to the left.
        right_rot (int):
            The amount of characters the string is to be rotated to the right.

    Returns:
        str:
            The rotated string.

    Example:
        ``rotate_string("HELLO", 1, 3)`` should return ``"LLOHE"``.
    """
    assert len(string) >= right_rot - left_rot
    # TODO
    result = string
    str_len = len(string)
    #print("len ist: ", str_len, "vom Typ", type(str_len))
    tmp = list(string)
    for i in range(left_rot):
        result = tmp[1:]
        result.append(tmp[0])
        #print("\nfuer i = ", i, "tmp= ", tmp, "result=", result)
        tmp = result
    for i in range(right_rot):
        result = tmp[:str_len - 1]
        result.insert(0, tmp[str_len - 1])
        #print("\nfuer i = ", i, "tmp= ", tmp, "result=", result)
        tmp = result
        # to ask: sollte das hier nicht wie Zeiger in C funktionieren ?
        # tmp zeigt auf result
        # dann verändert sich result, also result zeigt jetzt auf etwas 
        # Anderes, so habe ich erwartet, dass tmp auch auf das gleiche Datum zeigt. 
        # Ist aber nicht so, wieso ?
        
    return ''.join(result)


def rotationally_equivalent(string1, string2):
    """Checks if the strings are rotationally equivalent.

    Args:
        string1 (str):
            The first string to be checked.
        string2 (str):
            The second string to be checked.

    Returns:
        bool:
            True iff the strings are rotationally equivalent, False otherwise.
    """
    # TODO
    if len(string1) != len(string2):
        return False
    str_len = len(string1)
    if string1 == string2:
        return True
    for i in range(str_len):
        string2 = rotate_string(string2, 1, 0)
        if string1 == string2:
            return True
    return False


def rotationally_equivalent_2(string1, string2):
    """Checks if the strings are rotationally equivalent.

    Args:
        string1 (str):
            The first string to be checked.
        string2 (str):
            The second string to be checked.

    Returns:
        bool:
            True iff the strings are rotationally equivalent, False otherwise.
    """
    # TODO
    if len(string1) != len(string2):
        return False
    dic1 = count_char_frequency(string1)
    dic2 = count_char_frequency(string2)
    for key, value in dic1.items():
        if (value != dic2.get(key)):
            return False
    return True


def main():
    """The Main-Function of the programme. Is executed whenever this file is
    executed at top level.

    Hier kann beliebiger Testcode stehen, der bei der Korrektur vollständig
    ignoriert wird.
    """
    print("\nThe word contains <string> the character <s>: ", contains_char("string", "s"))
    print("The word contains <string> the character <p>: ", contains_char("string", "p"))

    print("\nThe word <oxo> ist palindrome: ", is_palindrome("oxo"))
    print("The word <string> ist palindrome: ", is_palindrome("string"))

    print("\ndictionnary of the Word <anticonstitutionnellement> is ", count_char_frequency("anticonstitutionnellement"))

    print("\ndictionnary of the Word <ecole> is ", count_char_frequency("ecole"))

    print("\nfirst repeting char in str <ecole> is: ", first_non_repeating_char("ecole", True))
    print("first non repeting char in str <ecole> is: ", first_non_repeating_char("ecole"))

    print("\nrotate_string('HELLO', 1, 3) ist: ", rotate_string("HELLO", 1, 3))
    print("rotate_string('HELLO', 2, 3) ist: ", rotate_string("HELLO", 2, 3))

    print("\nthe strings <HELLO> and <LHLEO> are rotationally equivalent (v_1) : ", rotationally_equivalent("HELLO", "LHLEO"))
    print("the strings <HELLO> and <LHLEO> are rotationally equivalent (v_2): ", rotationally_equivalent_2("HELLO", "LHLEO"))
    print("\nthe strings <HELLO> and <LLOHE> are rotationally equivalent (v_1) : ", rotationally_equivalent("HELLO", "LLOHE"))
    print("the strings <HELLO> and <LLOHE> are rotationally equivalent (v_2): ", rotationally_equivalent_2("HELLO", "LLOHE"))


if __name__ == "__main__": main()